<?php
namespace Inti\Config;

use ArrayObject;

/**
 * Configuration data.
 *
 * @package Inti\Config
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-config
 */
final class Config extends ArrayObject implements ConfigInterface
{
    /** @var bool */
    private $dirty = false;

    /**
     * ConfigArray constructor.
     *
     * @param array $input
     */
    public function __construct(array $input = [])
    {
        parent::__construct([], 0, 'ArrayIterator');
        $this->exchangeArray($input);
    }

    /**
     * Returns true if any of the data changed during runtime.
     *
     * @return bool
     */
    public function isDirty(): bool
    {
        if ($this->dirty) {
            return true;
        }

        foreach ($this as $value) {
            if ($value instanceof ConfigInterface && $value->isDirty()) {
                return true;
            }
        }

        return false;
    }

    /**
     * Returns the array copy of this object.
     *
     * @return array
     */
    public function getArrayCopy()
    {
        return array_map(function ($value) {
            return $value instanceof static ? $value->getArrayCopy() : $value;
        }, parent::getArrayCopy());
    }

    /**
     * Replaces the contents of this array with $input.
     *
     * @param array $input
     * @return array
     */
    public function exchangeArray($input)
    {
        return parent::exchangeArray(array_map(function ($value) {
            return is_array($value) ? new static($value) : $value;
        }, $input));
    }

    /**
     * @param string $index
     * @param mixed $newval
     */
    public function offsetSet($index, $newval)
    {
        $this->dirty = true;
        parent::offsetSet($index, $newval);
    }

    /**
     * @param string $index
     */
    public function offsetUnset($index)
    {
        $this->dirty = true;
        parent::offsetUnset($index);
    }

    /**
     * Returns all the values different from the given $other.
     *
     * @param ConfigInterface $other
     * @return ConfigInterface
     */
    public function diff(ConfigInterface $other): ConfigInterface
    {
        $diff = [];

        foreach ($this as $key => $value) {
            $compareTo = $other[$key];
            $localDiff = null;

            if ($value instanceof ConfigInterface) {
                $localDiff = $value->diff($compareTo);
            } else if ($value !== $compareTo) {
                $localDiff = $value;
            }

            if (isset($localDiff)) {
                $diff[$key] = $localDiff;
            }
        }

        return new static($diff);
    }

    /**
     * Returns a new config with values from the first overriden by $other.
     *
     * @param ConfigInterface $other
     * @return ConfigInterface
     */
    public function merge(ConfigInterface $other): ConfigInterface
    {
        $merged = [];

        foreach ($this as $key => $value) {
            if ($value instanceof ConfigInterface) {
                $merged[$key] = $value->merge($other[$key] ?? new Config());
            } else if (isset($other[$key])) {
                $merged[$key] = $other[$key];
            } else {
                $merged[$key] = $value;
            }
        }

        return new Config($merged);
    }
}
