<?php
namespace Inti\Config\Service;

use Inti\Config\Config;
use Inti\Config\ConfigInterface;
use Inti\Config\Writer\LocalFileWriter;
use Inti\Config\Writer\WriterInterface;
use Laminas\ConfigAggregator\ConfigAggregator;
use Laminas\ConfigAggregator\PhpFileProvider;

/**
 * Manages the configuration.
 *
 * @package Inti\Config\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-config
 */
class ConfigManager implements ConfigManagerInterface
{
    const CONTAINER_KEY = '*config*';

    /** @var string */
    private $autoload;
    /** @var callable[]|string[] */
    private $providers;
    /** @var WriterInterface */
    private $writer;
    /** @var ConfigInterface */
    protected $base;
    /** @var ConfigInterface */
    private $local;
    /** @var bool */
    private $autoFlush = false;
    /** @var ConfigInterface */
    protected $config;

    /**
     * ConfigManager constructor.
     *
     * @param string|null $autoload
     * @param callable[]|string[] $providers
     * @param WriterInterface|null $writer
     */
    public function __construct(string $autoload = null, array $providers = [], WriterInterface $writer = null)
    {
        $this->autoload = $autoload;
        $this->providers = $providers;
        $this->writer = $writer ?? $autoload ? new LocalFileWriter($autoload) : null;
    }

    /**
     * Destruktor.
     */
    public function __destruct()
    {
        if ($this->autoFlush) {
            $this->flush();
        }
    }

    /**
     * @return ConfigInterface
     */
    protected function getBaseConfig(): ConfigInterface
    {
        if ($this->base === null) {
            $providers = $this->providers;
            $providers[] = $this->getAutoloadProvider('{,*.}global.php');
            $aggregator = new ConfigAggregator(array_filter($providers));

            $config = $aggregator->getMergedConfig();
            $this->base = new Config($config);
        }

        return $this->base;
    }

    /**
     * @return ConfigInterface
     */
    private function getLocalConfig(): ConfigInterface
    {
        if ($this->local === null) {
            $aggregator = new ConfigAggregator(array_filter([
                $this->getAutoloadProvider('{,*.}local.php')
            ]));
            $config = $aggregator->getMergedConfig();

            $this->local = new Config($config);
        }

        return $this->local;
    }

    /**
     * Returns the merged configuration.
     *
     * @return ConfigInterface
     */
    public function getConfig(): ConfigInterface
    {
        if ($this->config === null) {
            $base = $this->getBaseConfig();
            $local = $this->getLocalConfig();

            $this->config = $base->merge($local);
        }

        return $this->config;
    }

    /**
     * @param bool $autoFlush
     */
    public function setAutoFlush(bool $autoFlush): void
    {
        $this->autoFlush = $autoFlush;
    }

    /**
     * Writes the configuration to storage.
     */
    public function flush(): void
    {
        if ($this->writer === null) {
            return;
        }

        $config = $this->getConfig();
        if (!$config->isDirty()) return;

        $base = $this->getBaseConfig();

        foreach ($config as $key => $value) {
            if ($value instanceof ConfigInterface && $value->isDirty()) {
                $diff = $value->diff($base[$key]);
                $this->writer->write($key, $diff);
            }
        }
    }

    /**
     * @param string $glob
     * @return callable|null
     */
    private function getAutoloadProvider(string $glob): ?callable
    {
        if ($this->autoload === null) {
            return null;
        }

        return new PhpFileProvider(sprintf('%s/%s', $this->autoload, $glob));
    }

    /**
     * @param string $offset
     * @return bool
     */
    public function offsetExists($offset)
    {
        $merged = $this->getConfig();
        return $merged->offsetExists($offset);
    }

    /**
     * @param string $offset
     * @return mixed
     */
    public function offsetGet($offset)
    {
        $merged = $this->getConfig();
        return $merged->offsetGet($offset);
    }

    /**
     * @param string $offset
     * @param mixed $value
     */
    public function offsetSet($offset, $value)
    {
        $merged = $this->getConfig();
        $merged->offsetSet($offset, $value);
    }

    /**
     * @param string $offset
     */
    public function offsetUnset($offset)
    {
        $merged = $this->getConfig();
        $merged->offsetUnset($offset);
    }
}
