<?php
namespace IntiTest\Config;

use Inti\Config\Config;
use Inti\Config\ConfigInterface;
use PHPUnit\Framework\TestCase;

/**
 * Class ConfigArrayTest
 *
 * @package IntiTest\Config
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-config
 */
final class ConfigTest extends TestCase
{
    public function testAllArraysAreConvertedToConfigArray()
    {
        $config = new Config([
            'key1' => [
                'item3' => [
                    'param1' => 'value1',
                ],
            ],
        ]);

        $this->assertInstanceOf(ConfigInterface::class, $config['key1']);
        $this->assertInstanceOf(ConfigInterface::class, $config['key1']['item3']);
    }

    public function testAllConfigArraysAreConvertedToArrays()
    {
        $config = new Config([
            'key1' => [
                'item3' => [
                    'param1' => 'value1',
                ],
            ],
        ]);

        $array = $config->getArrayCopy();

        $this->assertIsArray($array['key1']);
        $this->assertIsArray($array['key1']['item3']);
    }

    public function testIsDirtyWhenDirectValueChanged()
    {
        $config = new Config([
            'key' => 'value',
        ]);

        $this->assertFalse($config->isDirty());
        $config['key'] = 'new value';
        $this->assertTrue($config->isDirty());
    }

    public function testIsDirtyWhenKeyUnset()
    {
        $config = new Config([
            'key' => 'value',
        ]);

        $this->assertFalse($config->isDirty());
        unset($config['key']);
        $this->assertTrue($config->isDirty());
    }

    public function testIsDirtyWhenValueDirty()
    {
        $config = new Config([
            'parent' => [
                'child' => [
                    'key' => 'value',
                ],
            ],
        ]);

        $this->assertFalse($config->isDirty());
        $config['parent']['child']['key'] = 'new value';
        $this->assertTrue($config->isDirty());
    }

    public function testCanCalculateDiff()
    {
        $base = new Config(['key' => 'value', 'other' => true]);
        $changed = new Config(['key' => 'new value', 'other' => true]);

        $diff = $changed->diff($base);

        $this->assertEquals(['key' => 'new value'], $diff->getArrayCopy());
    }

    public function testCanCalculateDeepDiff()
    {
        $base = new Config(['config' => ['key' => 'value', 'other' => true]]);
        $changed = new Config(['config' => ['key' => 'new value', 'other' => true]]);

        $diff = $changed->diff($base);

        $this->assertEquals(['config' => ['key' => 'new value']], $diff->getArrayCopy());
    }

    public function testDiffWillNotContainEmptyArrays()
    {
        $base = new Config([
            'config' => [
                'key' => 'value',
                'other' => true,
            ],
            'values' => [
                'sub_values' => [
                    'sub_key1' => '',
                    'sub_key2' => '',
                ],
            ],
        ]);
        $changed = new Config(['config' => ['key' => 'new value']]);

        $diff = $changed->diff($base);

        $this->assertEquals(['config' => ['key' => 'new value']], $diff->getArrayCopy());
    }
}
